@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for Zutils - Utilities dealing with compressed files
-# Copyright (C) 2009-2018 Antonio Diaz Diaz.
+# Copyright (C) 2009-2019 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute and modify it.
@@ -53,6 +53,9 @@ cat in > -in- || framework_failure
 cat in.lz > -in-.lz || framework_failure
 cat in.lz > lz_only.lz || framework_failure
 cat in in in in in in > in6 || framework_failure
+bad0_lz="${testdir}"/zero_bad_crc.lz
+bad0_gz="${testdir}"/zero_bad_crc.gz
+bad1_lz="${testdir}"/test_bad_crc.lz
 fail=0
 test_failed() { fail=1 ; printf " $1" ; [ -z "$2" ] || printf "($2)" ; }
 
@@ -71,6 +74,7 @@ for i in ${extensions}; do
 		test_failed $LINENO $i
 done
 
+"${ZCAT}" -N -v -s "${testdir}"/zcat_vs.dat > /dev/null || test_failed $LINENO
 "${ZCAT}" -N < in > copy || test_failed $LINENO
 cmp in copy || test_failed $LINENO
 "${ZCAT}" -N < in.gz > copy || test_failed $LINENO
@@ -89,6 +93,18 @@ cmp in copy || test_failed $LINENO
 cmp in copy || test_failed $LINENO
 "${ZCAT}" -N in in.gz in.bz2 in.lz -- -in- -in-.lz > copy || test_failed $LINENO
 cmp in6 copy || test_failed $LINENO
+"${ZCAT}" -Nq in in.gz in.bz2 in.lz "${bad0_lz}" -- -in- -in-.lz > copy
+[ $? = 1 ] || test_failed $LINENO
+cmp in6 copy || test_failed $LINENO
+"${ZCAT}" -Nq "${bad1_lz}" -- -in-.lz in in.gz in.bz2 in.lz > copy
+[ $? = 1 ] || test_failed $LINENO
+cmp in6 copy || test_failed $LINENO
+"${ZCAT}" -N . || test_failed $LINENO
+"${ZCAT}" -N -r . > /dev/null || test_failed $LINENO
+"${ZCAT}" -N -r > /dev/null || test_failed $LINENO
+"${ZCAT}" -N -R . > /dev/null || test_failed $LINENO
+"${ZCAT}" -N -R > /dev/null || test_failed $LINENO
+
 "${ZCAT}" -Nq --format=, in.lz
 [ $? = 1 ] || test_failed $LINENO
 "${ZCAT}" -Nq --format=,lz in.lz
@@ -258,6 +274,9 @@ done
 "${ZGREP}" -N pin.tar -e "GNU" > /dev/null || test_failed $LINENO
 "${ZGREP}" -N "GNU" < pin.tar > /dev/null || test_failed $LINENO
 "${ZGREP}" -N -r "GNU" . > /dev/null || test_failed $LINENO
+"${ZGREP}" -N -r "GNU" > /dev/null || test_failed $LINENO
+"${ZGREP}" -N -R "GNU" . > /dev/null || test_failed $LINENO
+"${ZGREP}" -N -R "GNU" > /dev/null || test_failed $LINENO
 "${ZGREP}" -N "nx_pattern" -r . in > /dev/null && test_failed $LINENO
 "${ZGREP}" -N -e "GNU" in > /dev/null || test_failed $LINENO
 "${ZGREP}" -N "GNU" < in > /dev/null || test_failed $LINENO
@@ -275,6 +294,12 @@ done
 	test_failed $LINENO
 "${ZGREP}" -N -L "nx_pattern" in in.gz in.bz2 in.lz -- -in- > /dev/null &&
 	test_failed $LINENO
+"${ZGREP}" -Nq -l "01234567890" in "${bad1_lz}" in.lz && test_failed $LINENO
+"${ZGREP}" -Nq -l "01234567890" in "${bad1_lz}" in.lz pin.tar > /dev/null ||
+	test_failed $LINENO
+
+"${ZGREP}" -N "GNU" .
+[ $? = 1 ] || test_failed $LINENO
 "${ZGREP}" -N --bad-option 2> /dev/null
 [ $? = 2 ] || test_failed $LINENO
 "${ZGREP}" -N "GNU" -s nx_file
@@ -300,9 +325,20 @@ done
 "${ZTEST}" -N < in.gz || test_failed $LINENO
 "${ZTEST}" -N < in.bz2 || test_failed $LINENO
 "${ZTEST}" -N < in.lz || test_failed $LINENO
+"${ZTEST}" -N - < in.lz || test_failed $LINENO
 "${ZTEST}" -N - in.gz - < in.lz || test_failed $LINENO
 "${ZTEST}" -N --lz='lzip -q' < in.lz || test_failed $LINENO
 "${ZTEST}" -N -r . || test_failed $LINENO
+"${ZTEST}" -N -r || test_failed $LINENO
+"${ZTEST}" -N -R . || test_failed $LINENO
+"${ZTEST}" -N -R || test_failed $LINENO
+
+"${ZTEST}" -Nq in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz
+[ $? = 2 ] || test_failed $LINENO
+lines=`"${ZTEST}" -N in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
+[ "${lines}" -eq 2 ] || test_failed $LINENO
+lines=`"${ZTEST}" -Nv in.gz "${bad0_lz}" in.bz2 "${bad1_lz}" in.lz 2>&1 | wc -l`
+[ "${lines}" -eq 5 ] || test_failed $LINENO
 "${ZTEST}" -Nq < in
 [ $? = 2 ] || test_failed $LINENO
 dd if=in.lz bs=1000 count=1 2> /dev/null | "${ZTEST}" -N -q
@@ -336,58 +372,122 @@ cat in.gz > a.gz || framework_failure
 
 cat in.lz in.lz > a.lz || framework_failure
 "${ZUPDATE}" -Nq -f a.bz2 a.gz
-{ [ $? = 1 ] && [ -e a.bz2 ] && [ -e a.gz ] && [ -e a.lz ] ; } ||
-	test_failed $LINENO
+[ $? = 1 ] || test_failed $LINENO
+[ -e a.bz2 ] || test_failed $LINENO
+[ -e a.gz ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
-"${ZUPDATE}" -N a.bz2
-{ [ $? = 0 ] && [ ! -e a.bz2 ] && [ -e a.gz ] && [ -e a.lz ] ; } ||
-	test_failed $LINENO
+"${ZUPDATE}" -N a.bz2 || test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ -e a.gz ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
-"${ZUPDATE}" -N a.gz
-{ [ $? = 0 ] && [ ! -e a.bz2 ] && [ ! -e a.gz ] && [ -e a.lz ] ; } ||
-	test_failed $LINENO
+"${ZUPDATE}" -N a.gz || test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ ! -e a.gz ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
 cat in.bz2 > a.bz2 || framework_failure
 cat in.gz > a.gz || framework_failure
 "${ZUPDATE}" -Nq a.bz2 a.gz
-{ [ $? = 1 ] && [ ! -e a.bz2 ] && [ -e a.gz ] && [ -e a.lz ] ; } ||
-	test_failed $LINENO
+[ $? = 1 ] || test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ -e a.gz ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
 cat in.bz2 > a.bz2 || framework_failure
 cat in.gz > a.gz || framework_failure
-"${ZUPDATE}" -N -f -k a.bz2 a.gz
-{ [ $? = 0 ] && [ -e a.bz2 ] && [ -e a.gz ] && [ -e a.lz ] ; } ||
-	test_failed $LINENO
+"${ZUPDATE}" -N -f -k a.bz2 a.gz || test_failed $LINENO
+[ -e a.bz2 ] || test_failed $LINENO
+[ -e a.gz ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
 cat in.bz2 > a.bz2 || framework_failure
 cat in.gz > a.gz || framework_failure
-"${ZUPDATE}" -N -f a.bz2 a.gz
-{ [ $? = 0 ] && [ ! -e a.bz2 ] && [ ! -e a.gz ] && [ ! -e a ] &&
-  [ -e a.lz ] ; } || test_failed $LINENO
+"${ZUPDATE}" -N -f a.bz2 a.gz || test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ ! -e a.gz ] || test_failed $LINENO
+[ ! -e a ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
 cat in.bz2 > a.bz2 || framework_failure
-"${ZUPDATE}" -N -1 -q a.bz2
-{ [ $? = 0 ] && [ ! -e a.bz2 ] && [ -e a.lz ] ; } || test_failed $LINENO
+cat "${bad0_gz}" > b.gz || framework_failure
+cat in.gz > c.gz || framework_failure
+"${ZUPDATE}" -N -f a.bz2 b.gz c.gz 2> /dev/null && test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ -e b.gz ] || test_failed $LINENO
+[ -e c.gz ] || test_failed $LINENO
+[ ! -e a ] || test_failed $LINENO
+[ ! -e b ] || test_failed $LINENO
+[ ! -e c ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
+rm -f a.lz b.gz c.gz || framework_failure
+
+cat in.bz2 > a.bz2 || framework_failure
+"${ZUPDATE}" -N -1 -q a.bz2 || test_failed $LINENO
+[ ! -e a.bz2 ] || test_failed $LINENO
+[ -e a.lz ] || test_failed $LINENO
 rm -f a.lz || framework_failure
 
 mkdir tmp2
 mkdir tmp2/tmp3
 cat in.bz2 > tmp2/tmp3/a.bz2 || framework_failure
 cat in.gz > tmp2/tmp3/a.gz || framework_failure
-"${ZUPDATE}" -N -r --format=gz tmp2
-{ [ $? = 0 ] && [ -e tmp2/tmp3/a.bz2 ] && [ ! -e tmp2/tmp3/a.gz ] &&
-  [ -e tmp2/tmp3/a.lz ] ; } || test_failed $LINENO
+"${ZUPDATE}" -N -r --format=gz tmp2 || test_failed $LINENO
+[ -e tmp2/tmp3/a.bz2 ] || test_failed $LINENO
+[ ! -e tmp2/tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp2/tmp3/a.lz ] || test_failed $LINENO
 rm -f tmp2/tmp3/a.lz || framework_failure
-"${ZUPDATE}" -N -r --format=bz2 tmp2
-{ [ $? = 0 ] && [ ! -e tmp2/tmp3/a.bz2 ] && [ ! -e tmp2/tmp3/a.gz ] &&
-  [ -e tmp2/tmp3/a.lz ] ; } || test_failed $LINENO
+"${ZUPDATE}" -N -r --format=bz2 tmp2 || test_failed $LINENO
+[ ! -e tmp2/tmp3/a.bz2 ] || test_failed $LINENO
+[ ! -e tmp2/tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp2/tmp3/a.lz ] || test_failed $LINENO
+rm -f tmp2/tmp3/a.lz || framework_failure
+cat in.bz2 > tmp2/tmp3/a.bz2 || framework_failure
+cat in.gz > tmp2/tmp3/a.gz || framework_failure
+cd tmp2 || framework_failure
+"${ZUPDATE}" -N -r -k -f . || test_failed $LINENO
+[ -e tmp3/a.bz2 ] || test_failed $LINENO
+[ -e tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp3/a.lz ] || test_failed $LINENO
+rm -f tmp3/a.lz || framework_failure
+"${ZUPDATE}" -N -r -k -f || test_failed $LINENO
+[ -e tmp3/a.bz2 ] || test_failed $LINENO
+[ -e tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp3/a.lz ] || test_failed $LINENO
+rm -f tmp3/a.lz || framework_failure
+"${ZUPDATE}" -N -R -k -f . || test_failed $LINENO
+[ -e tmp3/a.bz2 ] || test_failed $LINENO
+[ -e tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp3/a.lz ] || test_failed $LINENO
+rm -f tmp3/a.lz || framework_failure
+"${ZUPDATE}" -N -R -k -f || test_failed $LINENO
+[ -e tmp3/a.bz2 ] || test_failed $LINENO
+[ -e tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp3/a.lz ] || test_failed $LINENO
+rm -f tmp3/a.lz || framework_failure
+"${ZUPDATE}" -N -r -f . || test_failed $LINENO
+[ ! -e tmp3/a.bz2 ] || test_failed $LINENO
+[ ! -e tmp3/a.gz ] || test_failed $LINENO
+[ -e tmp3/a.lz ] || test_failed $LINENO
+cd .. || framework_failure
 rm -r tmp2 || framework_failure
 
+if ln -s '.' slink 2> /dev/null ; then
+	"${ZCAT}" -N -r slink > /dev/null || test_failed $LINENO
+	"${ZGREP}" -N -r "GNU" slink > /dev/null || test_failed $LINENO
+	"${ZTEST}" -N -r slink || test_failed $LINENO
+	"${ZUPDATE}" -N -r -f slink || test_failed $LINENO
+else
+	printf "\nwarning: skipping link test: 'ln' does not work on your system."
+fi
+rm -f slink || framework_failure
+
 echo
 if [ ${fail} = 0 ] ; then
 	echo "tests completed successfully."
