@@ -1,5 +1,5 @@
 /*  Common code for zcat and zgrep
-    Copyright (C) 2010-2018 Antonio Diaz Diaz.
+    Copyright (C) 2010-2019 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ int open_instream( std::string & input_filename, const bool search,
   int infd = open( input_filename.c_str(), O_RDONLY | O_BINARY );
   if( infd < 0 )
     {
+    const int saved_errno = errno;
     if( search && simple_extension_index( input_filename ) < 0 )
       {
       for( int i = 0; i < num_formats; ++i )
@@ -51,7 +52,8 @@ int open_instream( std::string & input_filename, const bool search,
           }
       }
     if( infd < 0 && !no_messages )
-      show_error2( "Can't open input file", input_filename.c_str() );
+      show_file_error( input_filename.c_str(), "Can't open input file",
+                       saved_errno );
     }
   return infd;
   }
