@@ -1,5 +1,5 @@
 /*  Common code for zcmp and zdiff
-    Copyright (C) 2010-2018 Antonio Diaz Diaz.
+    Copyright (C) 2010-2019 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ int open_instream( const std::string & input_filename )
   {
   const int infd = open( input_filename.c_str(), O_RDONLY | O_BINARY );
   if( infd < 0 )
-    show_error2( "Can't open input file", input_filename.c_str() );
+    show_file_error( input_filename.c_str(), "Can't open input file", errno );
   return infd;
   }
 
