@@ -1,5 +1,5 @@
 /*  Zdiff - decompress and compare two files line by line
-    Copyright (C) 2010-2018 Antonio Diaz Diaz.
+    Copyright (C) 2010-2019 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ void show_help()
                "compressed, its decompressed content is used. Zdiff is a front end to\n"
                "the diff program and has the limitation that messages from diff refer to\n"
                "temporary filenames instead of those specified.\n"
-               "\nThe supported formats are bzip2, gzip, lzip and xz.\n"
+               "\nThe formats supported are bzip2, gzip, lzip and xz.\n"
                "\nUsage: zdiff [options] file1 [file2]\n"
                "\nZdiff compares file1 to file2. If file2 is omitted zdiff tries the\n"
                "following:\n"
@@ -141,7 +141,7 @@ bool set_fifonames( const std::string filenames[2] )
         if( mkfifo( fifonames[i].c_str(), S_IRUSR | S_IWUSR ) == 0 )
           continue;
         }
-      show_error2( "Can't create FIFO", fifonames[i].c_str() );
+      show_file_error( fifonames[i].c_str(), "Can't create FIFO", errno );
       return false;
       }
   return true;
@@ -419,7 +419,7 @@ int main( const int argc, const char * const argv[] )
   for( int i = 0; i < 2; ++i )
     if( filenames[i] != "-" && close( infd[i] ) != 0 )
       {
-      show_error2( "Can't close input file", filenames[i].c_str() );
+      show_file_error( filenames[i].c_str(), "Error closing input file", errno );
       retval = 2;
       }
 
